using System.Security.Cryptography.X509Certificates;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;

namespace FitConnect.Encryption;

public class CertificateHelper {
    private readonly ILogger? _logger;

    public CertificateHelper(ILogger? logger = null) {
        _logger = logger;
    }

    internal bool ValidateCertificate(JsonWebKey key) {
        var certificates = key.X5c.Select(s => new X509Certificate2(Convert.FromBase64String(s)))
            .ToList();
        var valid = certificates.Aggregate(true,
            (result, cert) => result
                              && cert.Verify()
                              && OcspCheck(cert, cert.Issuer)
        );
        return valid;
    }

    private bool OcspCheck(X509Certificate2 certificateX509, string issuer) {
        var issuerBytes = Convert.FromBase64String(issuer);

        var issuerX509 = new X509Certificate2(issuerBytes);
        var certificateChain = new X509Chain();
        certificateChain.ChainPolicy.RevocationMode = X509RevocationMode.Online;
        certificateChain.ChainPolicy.RevocationFlag = X509RevocationFlag.ExcludeRoot;
        certificateChain.ChainPolicy.VerificationFlags = X509VerificationFlags.NoFlag;
        certificateChain.ChainPolicy.ExtraStore.Add(issuerX509);
        certificateChain.Build(certificateX509);


        return certificateChain.ChainStatus.Length == 0;
    }
}
